/*

  sunplasma - fly or dive around in a simulated virtual world
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __RGBAMAP_HPP
#define __RGBAMAP_HPP

#include "fatal.hpp"
#include "tempmatrix.hpp"

/*
  if possible always use the standard:
  rgb 0 black  255 highlight
  alpha 0 transparent  255 opaque
*/

typedef uint8 rgba[4];
void appendtobinbuffer (const rgba &a, yyybinbuff::binbuff &b);
const yyybinbuff::binbuff::index readfrombinbuffer (rgba &a, const yyybinbuff::binbuff &b, const yyybinbuff::binbuff::index pos);


template<class A, class M>
inline void transform (tempmatrix<A> &a, const M &m)
{
  A *const p = a.getrawdatapointer();
  int count=a.getcount();
  for (int i=0;i<count;++i) m(p[i]);
};

template<class A, class M>
inline void transform (tempmatrix<A> &a, M &m)
{
  A *const p = a.getrawdatapointer();
  int count=a.getcount();
  for (int i=0;i<count;++i) m(p[i]);
};

template<class A, class B, class M>
inline void combine (tempmatrix<A> &a, const tempmatrix<B> &b, const M &m)
{
  if ((a.sizex()!=b.sizex()) || (a.sizey()!=b.sizey())) fatal( "combine<mat,mat,mod>: sizes mismatch");
  A *const p = a.getrawdatapointer();
  const B *const q = b.getrawdatapointer();
  int count=a.getcount();
  for (int i=0;i<count;++i) m(p[i],q[i]);
};


// function object converters

template <class R, class G, class B, class A>
struct apply4 {
  inline apply4 (R &r_, G &g_, B &b_, A &a_) : r(r_), g(g_), b(b_), a(a_) {};
  inline void operator() (rgba &v) const
  {
    r(v[0]);
    g(v[1]);
    b(v[2]);
    a(v[3]);
  };
private:
  R &r;
  G &g;
  B &b;
  A &a;
};

template <class R, class G, class B, class A>
struct apply4c {
  inline apply4c (const R &r_, const G &g_, const B &b_, const A &a_) : r(r_), g(g_), b(b_), a(a_) {};
  inline void operator() (rgba &v) const
  {
    r(v[0]);
    g(v[1]);
    b(v[2]);
    a(v[3]);
  };
private:
  const R &r;
  const G &g;
  const B &b;
  const A &a;
};


// transformers

/*struct scale {
  inline scale (const float s_) : s(s_) {};
  inline void operator () (float &v) const { v*=s; };
private:
  const float s;
  };*/

struct constant {
  inline constant (const uint8 s_) : s(s_) {};
  inline void operator () (uint8 &v) const { v=s; };
private:
  const uint8 s;
};

/*struct clamp {
  inline clamp (const float min_, const float max_)
    : min(min_), max(max_) {};
  inline void operator () (rgba &v) const
  {
    for (int i=0;i<4;++i) {
      if (v[i]<min) v[i]=min;
      else if (v[i]>max) v[i]=max;
    };
  };
private:
  const float min, max;
  };*/

/*struct threshhold {
  inline threshhold (const float split_,
		     const bool belowkill_, const float belowvalue_,
		     const bool abovekill_, const float abovevalue_)
    : split(split_),
      belowkill(belowkill_), belowvalue(belowvalue_),
      abovekill(abovekill_), abovevalue(abovevalue_) {};
  inline void operator () (float &v) const
  {
    if (v >= split) {
      if (abovekill) v=abovevalue;
    }else{
      if (belowkill) v=belowvalue;
    };
  };
private:
  const float split;
  const bool belowkill;
  const float belowvalue;
  const bool abovekill;
  const float abovevalue;
  };*/

/*struct identity { // nameclash with stl
  inline void operator () (const uint8 &) const {};
  };*/

/*struct statistics {
  inline statistics () { reset (); };
  inline void reset () { min=1e20; max=-1e20; avg=0; count=0; };
  inline void finish () { avg /= count; };
  inline const float getavg() const { return avg; };
  inline const float getmin() const { return min; };
  inline const float getmax() const { return max; };
  inline void operator () (const float &v)
  {
    avg += v;
    if (v<min) min=v;
    if (v>max) max=v;
    count++;
  };
private:
  double avg;
  float min, max;
  int count;
  };*/

/*struct linearmap {
  inline linearmap (const float m_, const float c_) : m(m_), c(c_) {};
  inline void operator () (float &v) const { v = m * v + c; };
private:
  float m,c;
  };*/

// combiners

struct blend {
  inline void operator () (rgba &a, const rgba &b) const
  {
    const uint16 weight_B = b[3];
    const uint16 weight_A = 255 - weight_B;

    for (int i=0;i<3;++i) a[i] = ((a[i]*weight_A) + (b[i]*weight_B)) >> 8;
  };
};

struct overwrite {
  void operator () (rgba &x, const uint8 &y) const { x[0]=x[1]=x[2]=x[3]=y; };
};

#endif
