/*

  yyyselector - a select() wrapper
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __SELECTOR_HPP
#define __SELECTOR_HPP

#include "yyyzeit.hpp"

#include <sys/types.h>

class selector {
public:
  selector ();
  selector (const selector &);
  selector & operator= (const selector &);

  void reset(); // clears read and write lists and sets timeout to infinity
  void add_read(const int handle);
  void add_write(const int handle);
  void add_exception(const int handle);
  void settimeout(const yyyzeit::zeit & timeout_); // "timeout" is the absolute timepoint of the timeout !
  void waitnow ();
  const bool isready_read(const int handle) const;
  const bool isready_write(const int handle) const;
  const bool isready_exception(const int handle) const;
  inline const bool istimeoutexpired() const { return timeout_expired; };
  // if the timeout has expired this does not mean that there are no ready read or write handles !

private:
  fd_set readfds, writefds, exceptionfds;
  bool infinity;
  yyyzeit::zeit timeout;
  bool timeout_expired;
};

#endif
