/*

  yyygtkscheduler - implementation of the coopscheduler concept using gtk
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "yyygtkscheduler.hpp"
#include <gtk/gtk.h>

using namespace yyygtkscheduler;

scheduler *scheduler::gsched = 0;

scheduler::scheduler ()
{
};

scheduler::~scheduler ()
{
  if (gsched == this) gsched = 0;
};



scheduler::genericnotify::genericnotify (scheduler *const sched_)
  : sched (sched_)
{
};



scheduler::ionotify::ionotify (scheduler *const sched_)
  : genericnotify(sched_), isactive_rep(false)
{
};

scheduler::ionotify::~ionotify ()
{
  setactive(false);
};

void scheduler::ionotify::setfilehandle (const int filehandle_, const watchtype_t watchtype_)
{
  setactive(false);
  filehandle_rep=filehandle_;
  watchtype_rep=watchtype_;
  setactive(true);
};

static void ionotrecv (gpointer data, gint source, GdkInputCondition cond)
{
  scheduler::ionotify *thiss = static_cast<scheduler::ionotify *> (data);
  thiss->fire.emit(*thiss);
};

void scheduler::ionotify::setactive(const bool isactive_)
{
  if (isactive()==isactive_) return;
  isactive_rep=isactive_;

  if (isactive()) {
    GdkInputCondition cond=GDK_INPUT_READ; // otherwise compiler makes stupid warning
    switch (watchtype_rep) {
    case scheduler::ionotify::READ: cond=GDK_INPUT_READ; break;
    case scheduler::ionotify::WRITE: cond=GDK_INPUT_WRITE; break;
    case scheduler::ionotify::EXCEPTION: cond=GDK_INPUT_EXCEPTION; break;
    };
    gdktag = gdk_input_add (filehandle_rep, cond, &ionotrecv, this);
  }else{
    gdk_input_remove (gdktag);
  };
};




scheduler::abstimer::abstimer (scheduler *const sched_)
  : genericnotify(sched_), isactive_rep(false)
{
};

scheduler::abstimer::~abstimer ()
{
  setactive(false);
};

void scheduler::abstimer::setfiretime (const yyyzeit::zeit &firetime_)
{
  setactive(false);
  firetime_rep=firetime_;
  setactive(true);
};

static gint abstimerrecv (gpointer data)
{
  scheduler::abstimer *thiss = static_cast<scheduler::abstimer *> (data);
  thiss->fire.emit(*thiss);
  return FALSE;
};

void scheduler::abstimer::setactive(const bool isactive_)
{
  if (isactive()==isactive_) return;
  isactive_rep=isactive_;

  if (isactive()) {
    guint32 interval = firetime().getmicroseconds(); // FIXME this can overflow
    
    gdktag = gtk_timeout_add (interval, &abstimerrecv, this);
  }else{
    gtk_timeout_remove (gdktag);
  };
};
