/*

  yyytypemagic - do some stuff on c++ types
  Copyright (C) 2000  Erik Thiele

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#ifndef __YYYTYPEMAGIC_HPP
#define __YYYTYPEMAGIC_HPP

namespace yyytypemagic {

  // this stuff is shamelessly stolen from http://www.boost.org/boost/detail/type_traits.hpp
  // many thanks to Steve Cleary, Beman Dawes, Howard Hinnant & John Maddock.
  // i didn't know such thing is possible in C++ ;)

  template <typename T> struct typeconvert {
    typedef const T addconst;
    typedef T removeconst;
    typedef T& addreference;
    typedef T removereference;
  };
  template <typename T> struct typeconvert<const T> {
    typedef const T addconst;
    typedef T removeconst;
    typedef const T& addreference;
    typedef const T removereference;
  };
  template <typename T> struct typeconvert<T&> {
    typedef const T& addconst;
    typedef T& removeconst;
    typedef T& addreference;
    typedef T removereference;
  };
  template <typename T> struct typeconvert<const T&> {
    typedef const T& addconst;
    typedef T& removeconst;
    typedef const T& addreference;
    typedef const T removereference;
  };

  // remove const from a type
  template <typename T> struct remove_const { typedef T type; };
  template <typename T> struct remove_const<const T> { typedef T type; };
  template <typename T> struct remove_const<const T&> { typedef T& type; };

  // add const to a type
  template <typename T> struct add_const { typedef const T type; };
  template <typename T> struct add_const<const T> { typedef const T type; };
  template <typename T> struct add_const<const T&> { typedef const T& type; };

  // * convert a type T to a non-reference if it is one - remove_reference<T>
  template <typename T> struct remove_reference{ typedef T type; };
  template <typename T> struct remove_reference<T&>{ typedef T type; };
  
  // * convert a type T to a reference unless it is one - add_reference<T>
  template <typename T> struct add_reference{ typedef T& type; };
  template <typename T> struct add_reference<T&>{ typedef T& type; };

};

#endif
